package net.hostsharing.hsadminng.hs.office.debitor;

import net.hostsharing.hsadminng.hs.office.contact.HsOfficeContactEntity;
import net.hostsharing.hsadminng.hs.office.partner.HsOfficePartnerEntity;
import net.hostsharing.hsadminng.hs.office.person.HsOfficePersonEntity;
import net.hostsharing.hsadminng.hs.office.person.HsOfficePersonType;
import net.hostsharing.hsadminng.hs.office.relationship.HsOfficeRelationshipEntity;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;

class HsOfficeDebitorEntityUnitTest {

    private HsOfficeRelationshipEntity givenDebitorRel = HsOfficeRelationshipEntity.builder()
            .relAnchor(HsOfficePersonEntity.builder()
                    .personType(HsOfficePersonType.LEGAL_PERSON)
                    .tradeName("some partner trade name")
                    .optionalPartner(HsOfficePartnerEntity.builder()
                            .partnerNumber(12345)
                            .build())
                    .build())
            .relHolder(HsOfficePersonEntity.builder()
                    .personType(HsOfficePersonType.LEGAL_PERSON)
                    .tradeName("some billing trade name")
                    .build())
            .contact(HsOfficeContactEntity.builder().label("some label").build())
            .build();

    @Test
    void toStringContainsPartnerAndContact() {
        final var given = HsOfficeDebitorEntity.builder()
                .debitorNumberSuffix((byte)67)
                .debitorRel(givenDebitorRel)
                .defaultPrefix("som")
                .build();

        final var result = given.toString();

        assertThat(result).isEqualTo("debitor(D-1234567: rel(relAnchor='LP some partner trade name', relHolder='LP some billing trade name'), som)");
    }

    @Test
    void toShortStringContainsDebitorNumber() {
        final var given = HsOfficeDebitorEntity.builder()
                .debitorRel(givenDebitorRel)
                .debitorNumberSuffix((byte)67)
                .build();

        final var result = given.toShortString();

        assertThat(result).isEqualTo("D-1234567");
    }

    @Test
    void getDebitorNumberWithPartnerNumberAndDebitorNumberSuffix() {
        final var given = HsOfficeDebitorEntity.builder()
                .debitorRel(givenDebitorRel)
                .debitorNumberSuffix((byte)67)
                .build();

        final var result = given.getDebitorNumber();

        assertThat(result).isEqualTo(1234567);
    }

    @Test
    void getDebitorNumberWithoutPartnerReturnsNull() {
        givenDebitorRel.getRelAnchor().setOptionalPartner(null);
        final var given = HsOfficeDebitorEntity.builder()
                .debitorRel(givenDebitorRel)
                .debitorNumberSuffix((byte)67)
                .build();

        final var result = given.getDebitorNumber();

        assertThat(result).isNull();
    }

    @Test
    void getDebitorNumberWithoutPartnerNumberReturnsNull() {
        givenDebitorRel.getRelAnchor().getOptionalPartner().setPartnerNumber(null);
        final var given = HsOfficeDebitorEntity.builder()
                .debitorRel(givenDebitorRel)
                .debitorNumberSuffix((byte)67)
                .build();

        final var result = given.getDebitorNumber();

        assertThat(result).isNull();
    }

    @Test
    void getDebitorNumberWithoutDebitorNumberSuffixReturnsNull() {
        final var given = HsOfficeDebitorEntity.builder()
                .debitorRel(givenDebitorRel)
                .debitorNumberSuffix(null)
                .build();

        final var result = given.getDebitorNumber();

        assertThat(result).isNull();
    }
}
